@@ -6,7 +6,7 @@ const app = angular.module('dataiku.code-studios', [])
 
 app.controller("CodeStudiosCommonController", function($scope, $state, $rootScope, $stateParams, $q, $controller, TopNav, CodeStudiosService, LoggerProvider, WT1,
                                                     DataikuAPI, FutureWatcher, Dialogs, ActivityIndicator, TAIL_STATUS, DKUtils, StateUtils, PluginsService,
-                                                    ActiveProjectKey, FutureProgressModal, CreateModalFromTemplate) {
+                                                    ActiveProjectKey, FutureProgressModal, CreateModalFromTemplate, SpinnerService) {
 
     $scope.hooks = $scope.hooks || {};
 
@@ -54,13 +54,24 @@ app.controller("CodeStudiosCommonController", function($scope, $state, $rootScop
         return deferred.promise;
     }
 
+    function handlePullResponse(response) {
+        const result = response.data;
+        if (result && result.jobId && !result.hasResult) {
+            return FutureWatcher.watchJobId(result.jobId)
+                .then(data => data);
+        } else {
+            return result;
+        }
+    }
+
     function doForcePullEverything(codeStudioObject) {
         WT1.event("code_studio_action_syncstop", {type:codeStudioObject.templateId});
         let deferred = $q.defer();
         DataikuAPI.codeStudioObjects.getState($stateParams.projectKey, codeStudioObject.id).success(function(result) {
             let syncedZones = result.syncedZones;
             const promises = syncedZones.map(z => {
-                return DataikuAPI.codeStudioObjects.pullBundle($stateParams.projectKey, codeStudioObject.id, z.id, true);
+                return DataikuAPI.codeStudioObjects.pullBundle($stateParams.projectKey, codeStudioObject.id, z.id, true)
+                    .then(handlePullResponse);
             });
             $q.all(promises)
                 .then(responses => {
@@ -210,27 +221,32 @@ app.controller("CodeStudiosCommonController", function($scope, $state, $rootScop
     $scope.doPull = function(projectKey, codeStudioObjectId, selectedOptions) {
         let deferred = $q.defer();
         const promises = selectedOptions.map(opt => {
-            return DataikuAPI.codeStudioObjects.pullBundle(projectKey, codeStudioObjectId, opt.name, true);
+            return DataikuAPI.codeStudioObjects.pullBundle(projectKey, codeStudioObjectId, opt.name, true)
+                .then(handlePullResponse);
         });
         //if we are in the kubikle runtime page, we have to handle the UI state
         //that we don't have from the kubikles list page
         if ($scope.uiState) {
             delete $scope.uiState['pullSyncOK'];
             $scope.uiState.syncing = 'pull';
-            $q.all(promises)
-                .then(responses => {
-                    $scope.uiState['pullSyncOK'] = true;
-                    setTimeout(() => delete $scope.uiState['pullSyncOK'], 2000);
-                    $scope.uiState['pullSyncLog'] = `Last sync ${(new Date()).toLocaleTimeString()}`;
-                    deferred.resolve(responses);
-                }, error => {
-                    ActivityIndicator.error("Synchronization failed");
-                    $scope.uiState['pullSyncOK'] = false;
-                    $scope.uiState['pullSyncLog'] = `Last sync ${(new Date()).toLocaleTimeString()} failed: ${error.message}`;
-                    deferred.reject(error);
-                }).finally(() => { $scope.uiState.syncing = null; });
+            SpinnerService.lockOnPromise(
+                $q.all(promises)
+                    .then(responses => {
+                        $scope.uiState['pullSyncOK'] = true;
+                        setTimeout(() => delete $scope.uiState['pullSyncOK'], 2000);
+                        $scope.uiState['pullSyncLog'] = `Last sync ${(new Date()).toLocaleTimeString()}`;
+                        deferred.resolve(responses);
+                    }, error => {
+                        ActivityIndicator.error("Synchronization failed");
+                        $scope.uiState['pullSyncOK'] = false;
+                        $scope.uiState['pullSyncLog'] = `Last sync ${(new Date()).toLocaleTimeString()} failed : ${error.message}`;
+                        deferred.reject(error);
+                    }).finally(() => { $scope.uiState.syncing = null; })
+            );
         } else {
-            $q.all(promises).then(deferred.resolve, deferred.reject);
+            SpinnerService.lockOnPromise(
+                $q.all(promises).then(deferred.resolve, deferred.reject)
+            );
         }
         return deferred.promise;
     }
@@ -526,7 +542,7 @@ app.controller("CodeStudiosListController", function($scope, $controller, $state
         if (stoppedCodeStudios.length > 0) {
             Promise
                 .all(stoppedCodeStudios.map(async k => $scope.restart(k.id)))
-                .then($scope.list, $scope.list);
+                .then(() => refreshList(), () => refreshList());
         }
     };
 
@@ -538,13 +554,22 @@ app.controller("CodeStudiosListController", function($scope, $controller, $state
                 .then(() => {
                     Promise
                         .all(runningCodeStudios.map(async k => $scope.doStop(k)))
-                        .then($scope.list, $scope.list);
+                        .then(() => refreshList(), () => refreshList());
                 });
         } else if (runningCodeStudios.length === 1) {
             $scope.stopCodeStudio(runningCodeStudios[0]);
         }
     };
 
+    function refreshList(attemptsLeft = 8) {
+        return $scope.list().then(() => {
+            if (!$scope.listItems.some(cs => cs.uiState.state === 'STARTING' || cs.uiState.state === 'STOPPING')) {
+                return new Promise(resolve => setTimeout(resolve, 200))
+                    .then(() => refreshList(attemptsLeft -1));
+            }
+        }) 
+    }
+
     $scope.gotoLogs = function(codeStudioObjectId, tab){
         $state.go('projects.project.code-studios.code-studio.logs', {
             codeStudioObjectId: codeStudioObjectId,
@@ -1382,11 +1407,56 @@ app.service('CodeStudiosService', function($rootScope, DataikuAPI, CreateModalFr
                     "projects.project.code-studios.list"
                 )
             }
-            DataikuAPI.codeStudioObjects.listHeads($stateParams.projectKey).success(function(data) {
-                newScope.codeStudioObjects = data.items.filter(e => !e.templateWasDeleted || e.uiState.state !== 'STOPPED');
-            }).error(setErrorInScope.bind(scope));
+
             newScope.selectedCodeStudioId = localStorageService.get(CSIO_FAVORITE_EDITOR_KEY);
 
+            DataikuAPI.codeStudioObjects.listHeads($stateParams.projectKey).success(function(data) {
+                let codeStudioExists = false;
+
+                const templateSyncsZone = (cso) => cso.syncedZones.find(synced => synced.zone == zone);
+
+                const usableCodeStudios = data.items
+                    .filter(item => !item.templateWasDeleted || item.uiState.state !== 'STOPPED');
+
+                const syncingCodeStudios = usableCodeStudios
+                    .filter(item => templateSyncsZone(item))
+                    .map(item => {
+                        if (item.id == newScope.selectedCodeStudioId) {
+                            codeStudioExists = true;
+                        }
+                        return {
+                            name: item.name,
+                            templateLabel: item.desc.label,
+                            templateId: item.templateId,
+                            id: item.id
+                        }
+                    }
+                );
+
+                // append templates not synchronizing selected zone so the user knows what to do
+                const nonSynchronizingTemplates = data.items.filter(item => !templateSyncsZone(item))
+                    .reduce((acc, item) => {
+                        const label = item.desc.label;
+                        if (!acc[label]) {
+                            acc[label] = {
+                                name: `This Code Studio template does not synchronize ${zone} files`,
+                                templateLabel: label + ' - unavailable',
+                                templateId: item.templateId,
+                                id: item.id,
+                                disabled: true
+                            };
+                        }
+                        return acc;
+                    }, {});
+
+                newScope.codeStudioObjects = Object.values(nonSynchronizingTemplates).concat(syncingCodeStudios);
+
+                if (!codeStudioExists) {
+                    newScope.selectedCodeStudioId = (syncingCodeStudios.length > 0) ? syncingCodeStudios[0].id : null;
+                }
+
+            }).error(setErrorInScope.bind(scope));
+
             newScope.go = function() {
                 localStorageService.set(CSIO_FAVORITE_EDITOR_KEY, newScope.selectedCodeStudioId);
                 const selectedCodeStudio = newScope.codeStudioObjects.find(e => e.id === newScope.selectedCodeStudioId);
@@ -1406,4 +1476,4 @@ app.service('CodeStudiosService', function($rootScope, DataikuAPI, CreateModalFr
 });
 
 
-})();
\ No newline at end of file
+})();
