@@ -9,14 +9,22 @@
      * Everything to store and manipulate dashboard filters.
      * Was previously located here: src/main/platypus/static/dataiku/js/dashboards/core.js
      */
-    app.factory('DashboardFilters', function(CHART_FILTERS, FILTERABLE_INSIGHT_TYPES, DashboardFiltersUrlParams, ChartFilterUtils, FiltersPanelPosition, ActivityIndicator, ChartTypeChangeHandler, ChartFilters, FilterFacetsService, Logger, DataikuAPI, ChartUsableColumns, FilterPanelContext, $stateParams, CachedAPICalls, DashboardPageUtils, TileUtils) {
-        const FILTERS_ON_MSG = 'Data is filtered';
+    app.factory('DashboardFilters', function($rootScope, CHART_FILTERS, FILTERABLE_INSIGHT_TYPES, DashboardFiltersUrlParams, ChartFilterUtils, FiltersPanelPosition, ActivityIndicator, ChartTypeChangeHandler, ChartFilters, FilterFacetsService, Logger, DataikuAPI, ChartUsableColumns, MonoFuture, FilterPanelContext, $stateParams, CachedAPICalls, DashboardPageUtils, TileUtils, translate) {
+        const FILTERS_ON_MSG = translate('DASHBOARD.FILTERS.STATUS.DATA_FILTERED', 'Data is filtered');
         const SAMPLING_PROPS = ['refreshableSelection', 'engineType'];
         const MAX_DATA_BYTES = -1; // default to no memory limit (for sampling)
         const filtersSourceTypes = Object.freeze({ DATASET: 'dataset', TILE: 'tile' });
 
         /** @type {Map<string, ChartFilter[]>} */
         const globalFiltersByPageId = new Map();
+        /** @type {Map<string, ChartFilter[]>} */
+        const dashboardFiltersFacetsByPageId = new Map();
+        /** @type {Map<string, ChartFilter[]>} */
+        const insightsFiltersFacetsByInsightId = new Map();
+        /** @type {Map<string, boolean>} */
+        const insightAllFiltersFacetsByInsightId = new Map();
+        /** @type {Map<string, boolean>} */
+        const insightAllFiltersByInsightId = new Map();
         /** @type {Map<string, boolean>} */
         const enableCrossFiltersByPageId = new Map();
         /** @type {Map<string, boolean>} */
@@ -57,6 +65,8 @@
 
         function resetAllDashboardPagesFilterPanelStates() {
             globalFiltersByPageId.clear();
+            dashboardFiltersFacetsByPageId.clear();
+            insightsFiltersFacetsByInsightId.clear();
             isFilterPanelDeactivatedByPageId.clear();
             enableCrossFiltersByPageId.clear();
         }
@@ -82,6 +92,57 @@
             return globalFiltersByPageId.get(pageId);
         }
 
+        function setDashboardFiltersFacets(filters, pageId) {
+            if (filters !== dashboardFiltersFacetsByPageId.get(pageId) && pageId) {
+                dashboardFiltersFacetsByPageId.set(pageId, filters);
+            }
+        }
+
+        function getDashboardFiltersFacets(pageId) {
+            return dashboardFiltersFacetsByPageId.get(pageId);
+        }
+
+        function setAllInsightFiltersFacets(filters, insightId) {
+            if (filters !== insightAllFiltersFacetsByInsightId.get(insightId) && insightId) {
+                insightAllFiltersFacetsByInsightId.set(insightId, filters);
+            }
+        }
+
+        // Return both insight + dashboard facets for this insight
+        function getAllInsightFiltersFacets(insightId) {
+            if(insightAllFiltersFacetsByInsightId.get(insightId)) {
+                return insightAllFiltersFacetsByInsightId.get(insightId);
+            }
+            return [];
+        }
+
+        function setAllInsightFilters(filters, insightId) {
+            if (filters !== insightAllFiltersByInsightId.get(insightId) && insightId) {
+                insightAllFiltersByInsightId.set(insightId, filters);
+            }
+        }
+
+        // Return both insight + dashboard filters for this insight
+        function getAllInsightFilters(insightId) {
+            if(insightAllFiltersByInsightId.get(insightId)) {
+                return insightAllFiltersByInsightId.get(insightId);
+            }
+            return [];
+        }
+
+        function setInsightsFiltersFacets(filters, insightId) {
+            if (filters !== insightsFiltersFacetsByInsightId.get(insightId) && insightId) {
+                insightsFiltersFacetsByInsightId.set(insightId, filters);
+            }
+        }
+
+        function getInsightsFiltersFacets(insightId) {
+            if(insightsFiltersFacetsByInsightId.get(insightId)) {
+                return insightsFiltersFacetsByInsightId.get(insightId);
+            }
+            return [];
+        }
+
         function setInsightsMap(newInsightsMap) {
             insightsMap = newInsightsMap;
         }
@@ -94,7 +155,7 @@
             const filterableTiles = [];
             const dashboardPageIdx = findDashboardPageIdx(dashboardPageId, dashboardPages);
             if (dashboardPageIdx > -1 && insightsMap) {
-                TileUtils.getFlattenTileList(dashboardPages[dashboardPageIdx].grid.tiles).forEach(tile => {
+                TileUtils.getFlattenedTileList(dashboardPages[dashboardPageIdx].grid.tiles).forEach(tile => {
                     if (FILTERABLE_INSIGHT_TYPES.includes(tile.insightType)) {
                         const mappedInsight = insightsMap[tile.insightId];
                         if (!mappedInsight) {
@@ -208,11 +269,11 @@
                 return '';
             }
             if (tileData.sourceDataset !== filtersParams.datasetSmartName) {
-                return 'datasets are different';
+                return translate('DASHBOARD.FILTERS.STATUS.UNUSABLE_DIFFERENT_DATASETS', 'datasets are different');
             }
             // no sense on checking for samples if datasets are different
             if (!isSameSampling(tileData.sampleSettings.selection, filtersParams.refreshableSelection.selection)) {
-                return 'samples are different';
+                return translate('DASHBOARD.FILTERS.STATUS.UNUSABLE_DIFFERENT_SAMPLES', 'samples are different');
             }
             return '';
         }
@@ -228,27 +289,26 @@
             const isPluralColumns = unusableFilters.length > 1 ? 's' : '';
             const isPluralOthers = otherColumnsLength > 1 ? 's' : '';
             const hasDifferentReasons = unusableFilters.slice(1).some(uf => uf.reason !== unusableFilters[0].reason);
-            return `${FILTERS_ON_MSG} - except for column${isPluralColumns} ${columnsNames}\
-                    ${otherColumnsLength > 0 ? ` + ${otherColumnsLength} other${isPluralOthers}` : ''} (\
-                        ${hasDifferentReasons ? 'various reasons' : unusableFilters[0].reason}\
-                        ${hasWarningRootCause ? `as ${warningRootCause}` : ''}\
+            return `${FILTERS_ON_MSG} - ${translate('DASHBOARD.FILTERS.STATUS.EXCEPT_COLUMN', `except for column${isPluralColumns}`, { columnCount: unusableFilters.length })} ${columnsNames}\
+                    ${otherColumnsLength > 0 ? ` + ${translate('DASHBOARD.FILTERS.STATUS.OTHER_COLUMNS', `${otherColumnsLength} other${isPluralOthers}`, { otherColumnCount: otherColumnsLength })}` : ''} (\
+                        ${hasDifferentReasons ? translate('DASHBOARD.FILTERS.STATUS.UNUSABLE_VARIOUS_REASONS', 'various reasons') : unusableFilters[0].reason}\
+                        ${hasWarningRootCause ? `${translate('DASHBOARD.FILTERS.STATUS.AS', 'as')} ${warningRootCause}` : ''}\
                     )`;
         }
 
         function setFilteringMessages(tileData, filtersParams) {
-            if (!tileData.filteringStatus.isInsightFiltered) {
-                tileData.filteringStatus.warningMessage = '';
-                tileData.filteringStatus.infoMessage = '';
+            if (!tileData.tileFilteringInformation.isInsightFiltered) {
+                tileData.tileFilteringInformation.warningMessage = '';
+                tileData.tileFilteringInformation.infoMessage = '';
                 return;
             }
             const inconsistentFilteringCause = getInconsistentFilteringCause(tileData, filtersParams);
             if (!inconsistentFilteringCause.length && !tileData.unusableColumnsFilters.length) {
-                tileData.filteringStatus.warningMessage = ''; // no warning message if no inconsistent filtering detected cause detected
-                tileData.filteringStatus.infoMessage = FILTERS_ON_MSG;
+                tileData.tileFilteringInformation.warningMessage = ''; // no warning message if no inconsistent filtering detected cause detected
             } else {
-                tileData.filteringStatus.warningMessage = getUnusableFiltersMessage(tileData.unusableColumnsFilters, inconsistentFilteringCause);
-                tileData.filteringStatus.infoMessage = tileData.filteringStatus.warningMessage.length > 0 ? ''
-                    : `${FILTERS_ON_MSG} - risk of inconsistency (${inconsistentFilteringCause})`;
+                tileData.tileFilteringInformation.warningMessage = getUnusableFiltersMessage(tileData.unusableColumnsFilters, inconsistentFilteringCause);
+                tileData.tileFilteringInformation.infoMessage = tileData.tileFilteringInformation.warningMessage.length > 0 ? ''
+                    : `${FILTERS_ON_MSG} - ${translate('DASHBOARD.FILTERS.STATUS.RISK_INCONSISTENCY', `risk of inconsistency (${inconsistentFilteringCause})`, { reason: inconsistentFilteringCause })}`;
             }
         }
 
@@ -293,17 +353,17 @@
             if (filter && usableColumns && usableColumns.length) {
                 const columnFound = usableColumns.find(e => e.column === filter.column);
                 if (columnFound && columnFound.type !== filter.type) {
-                    return 'type mismatch';
+                    return translate('DASHBOARD.FILTERS.STATUS.UNUSABLE_TYPE_MISMATCH', 'type mismatch');
                 }
             }
-            return 'not found';
+            return translate('DASHBOARD.FILTERS.STATUS.UNUSABLE_NOT_FOUND', 'not found');
         }
 
         function getUnusableColumnFilters(filters, usableColumns) {
             if (usableColumns && usableColumns.length && filters && filters.length) {
                 return filters
                     .filter(filter => !hasUsableColumn(usableColumns, filter))
-                    .map(filter => ({ columnName: filter.column, reason: getReason(filter, usableColumns) }));
+                    .map(filter => ({ columnName: ChartFilterUtils.isExplicitFilter(filter) ? filter.explicitConditions.flatMap(it => it.column) : filter.column, reason: getReason(filter, usableColumns) }));
             }
             return [];
         }
@@ -327,22 +387,23 @@
         }
 
         function onApplyFiltersSuccess(tileData, filtersParams, response) {
-            tileData.filteringStatus.isApplyingFilters = false;
-            tileData.filteringStatus.isInsightFiltered = hasGlobalFiltersApplied(response, tileData.appliedFilters);
+            tileData.tileFilteringInformation.isApplyingFilters = false;
+            tileData.tileFilteringInformation.isInsightFiltered = hasGlobalFiltersApplied(response, tileData.appliedFilters);
+            updateFiltersDetails(tileData.tileFilteringInformation.filtersDetails, response);
             setFilteringMessages(tileData, filtersParams);
         }
 
-        function onApplyFiltersError(error, filteringStatus, loadingData) {
-            filteringStatus.isApplyingFilters = false;
+        function onApplyFiltersError(error, tileFilteringInformation, loadingData) {
+            tileFilteringInformation.isApplyingFilters = false;
 
             if (!error || !error.data) {
                 return;
             }
             // If the error contains a filtered out message, the insight is considered as filtered (empty state)
-            filteringStatus.isInsightFiltered = error.data.message.includes('filtered out');
+            tileFilteringInformation.isInsightFiltered = error.data.message.includes('filtered out');
 
 
-            if (!filteringStatus.isInsightFiltered && error.data.message.includes('To finish your chart')) {
+            if (!tileFilteringInformation.isInsightFiltered && error.data.message.includes('To finish your chart')) {
                 loadingData.unconfiguredRoutine();
             }
         }
@@ -361,11 +422,14 @@
 
                 tileData.appliedFilters = angular.copy(matchingFilters);
                 tileData.unusableColumnsFilters = getUnusableColumnFilters(filters, tileData.usableColumns);
-                tileData.filteringStatus.isApplyingFilters = !!matchingFilters && !!matchingFilters.length;
+                tileData.tileFilteringInformation.isApplyingFilters = !!matchingFilters && !!matchingFilters.length;
+                // we cannot use matching filters as their min/max values may have changed and compromised the tooltip
+                tileData.tileFilteringInformation.filtersDetails = createFiltersDetails(filters.filter(filter => isFilterEffective(filter, filtersParams)));
+
                 return tileData.applyFilters(matchingFilters, loadingData)
                     .then(data => tileData.parseResponse ? onApplyFiltersSuccess(tileData, filtersParams, tileData.parseResponse(data)) : data)
                     .catch((error) => {
-                        onApplyFiltersError(error, tileData.filteringStatus, loadingData);
+                        onApplyFiltersError(error, tileData.tileFilteringInformation, loadingData);
                         // Let the insight handle the error
                         throw error;
                     });
@@ -476,6 +540,80 @@
             return explorationFilters;
         }
 
+        /**
+         * Get charts filters from explorations filters
+         * @param  {List<FilterElement.java>} filtersElements - Filters to convert from
+         * @return {List<ChartFilter.java>}
+         */
+        function getChartsFiltersFromFiltersElements(filtersElements) {
+            if (!filtersElements || !filtersElements.length) {
+                return [];
+            }
+            const chartsFilters = [];
+            filtersElements.forEach(filter => {
+                const chartFilter = {
+                    active: filter.active,
+                    column: filter.column,
+                    columnType: filter.facet.columnType,
+                    filterType: filter.facet.currentMode === 'EXPLICIT' ? 'EXPLICIT' : filter.facet.currentMode + '_FACET',
+                    minValue: filter.facet.minValue,
+                    maxValue: filter.facet.maxValue,
+                    isAGlobalFilter: filter.facet.isAGlobalFilter ?? false,
+                    isA: 'filter',
+                    includeEmptyValues: filter.includeEmptyValues || filter.facet.includeEmptyValues,
+                    label: filter.column
+                };
+
+                if(filter.facet.currentMode === 'EXPLICIT') {
+                    chartFilter.useMinimalUi = true;
+                }
+
+                if (filter.facet.explicitConditions) {
+                    chartFilter.explicitConditions = filter.facet.explicitConditions;
+                }
+                if (filter.facet.explicitExclude) {
+                    chartFilter.explicitExclude = filter.facet.explicitExclude;
+                }
+
+                if (filter.facet.selectedValues) {
+                    chartFilter.excludeOtherValues = true;
+                    chartFilter.selectedValues = filter.facet.selectedValues;
+                } else {
+                    chartFilter.excludedValues = filter.facet.excludedValues;
+                }
+
+                if (filter.alphanumFilter.selectedValues && filter.alphanumFilter.selectedValues.length > 0) {
+                    chartFilter.customOptions = {
+                        matchingMode: filter.alphanumFilter.params.mode.toUpperCase(),
+                        normalizationMode: filter.alphanumFilter.params.normalization.toUpperCase()
+                    };
+                    // overrides values of the facet
+                    const newValues = filter.alphanumFilter.selectedValues.reduce((acc, val) => {
+                        acc[val] = true;
+                        return acc;
+                    }, {});
+                    chartFilter.selectedValues = newValues;
+                    chartFilter.filterSelectionType = 'CUSTOM';
+                    chartFilter.filterType = 'ALPHANUM';
+                }
+
+                if (filter.facet.dateFilterType === CHART_FILTERS.DATE_TYPES.RANGE) {
+                    chartFilter.dateFilterType = filter.facet.dateFilterType;
+                } else if (filter.facet.dateFilterType === CHART_FILTERS.DATE_TYPES.RELATIVE) {
+                    chartFilter.dateFilterType = filter.facet.dateFilterType;
+                    chartFilter.dateFilterPart = filter.facet.dateFilterPart;
+                    chartFilter.dateFilterOption = filter.facet.dateFilterRelativeOption;
+                    chartFilter.dateFilterRelativePreset = filter.facet.dateFilterRelativePreset;
+                } else if (filter.facet.dateFilterType === CHART_FILTERS.DATE_TYPES.PART) {
+                    chartFilter.dateFilterType = filter.facet.dateFilterType;
+                    chartFilter.dateFilterPart = filter.facet.dateFilterPart;
+                }
+
+                chartsFilters.push(chartFilter);
+            });
+            return chartsFilters;
+        }
+
         /**
          * Computes the initial state of a dashboard slide filters, store them in the service and returns them.
          * @param {DashboardPage} page
@@ -598,7 +736,7 @@
         }
 
         function acceptColumnAsFilter(data) {
-            const ret = ChartTypeChangeHandler.stdAggregatedAcceptDimension(data);
+            const ret = ChartTypeChangeHandler.stdAggregatedAcceptDimensionOrHierarchy(data);
             return ret.accept && data.type !== 'GEOMETRY' && data.type !== 'GEOPOINT';
         }
 
@@ -744,6 +882,21 @@
             lowerCaseStatusByDatasetKey.clear();
         }
 
+        function updateFiltersDetails(filtersDetails, pivotResponse) {
+            // this is mainly for datasets filters facets
+            filtersDetails.insightsFiltersFacets = pivotResponse.filterFacets.filter(f => !f.isAGlobalFilter);
+            filtersDetails.allFiltersFacets = pivotResponse.filterFacets;
+        }
+
+        function createFiltersDetails(filters) {
+            return {
+                insightsFiltersFacets: [],
+                insightFilters: [],
+                allFilters: [...filters] ,
+                allFiltersFacets: []
+            };
+        }
+
         return {
             SAMPLING_PROPS,
             MAX_DATA_BYTES,
@@ -781,7 +934,16 @@
             isFiltersPanelDocked,
             hasCaseInsensitiveColumnNames,
             flushDatasetLowerCaseStatus,
-            resetAllDashboardPagesFilterPanelStates
+            resetAllDashboardPagesFilterPanelStates,
+            setDashboardFiltersFacets,
+            getDashboardFiltersFacets,
+            setInsightsFiltersFacets,
+            getInsightsFiltersFacets,
+            getChartsFiltersFromFiltersElements,
+            getAllInsightFiltersFacets,
+            getAllInsightFilters,
+            setAllInsightFilters,
+            setAllInsightFiltersFacets
         };
     });
 })();
