@@ -221,7 +221,8 @@
                                     summary: chart.summary,
                                     theme: chart.theme || DefaultDSSVisualizationTheme,
                                     customMeasures: $scope.explore.customMeasures,
-                                    reusableDimensions: $scope.explore.reusableDimensions
+                                    reusableDimensions: $scope.explore.reusableDimensions,
+                                    hierarchies: $scope.explore.hierarchies
                                 }
                             };
                             if (insight.params.refreshableSelection == null) {
@@ -284,6 +285,7 @@
 
                             $scope.addCustomMeasuresToScopeAndCache($scope.explore.customMeasures);
                             $scope.addBinnedDimensionToScopeAndCache($scope.explore.reusableDimensions);
+                            $scope.addHierarchiesToScopeAndCache($scope.explore.hierarchies);
 
 
                             Logger.info("Explore loaded, get summary");
