@@ -2,7 +2,8 @@
 'use strict';
 const app = angular.module('dataiku.agents', ['dataiku.ml.report', 'dataiku.lambda']);
 
-app.controller("_AgentSavedModelController", function($scope, $rootScope, $state, $controller, $stateParams, DataikuAPI, ActiveProjectKey, FullModelLikeIdUtils, $q, DKUtils, Dialogs, WT1) {
+app.controller("AgentSavedModelController", function($scope, $rootScope, $state, $controller, $stateParams, DataikuAPI, ActiveProjectKey, FullModelLikeIdUtils, $q,
+    DKUtils, Dialogs, WT1, Debounce) {
     $controller("_SavedModelReportController", {$scope});
 
     DataikuAPI.savedmodels.llmCommon.getStatus(ActiveProjectKey.get(), $stateParams.smId).success(function(data){
@@ -11,8 +12,9 @@ app.controller("_AgentSavedModelController", function($scope, $rootScope, $state
 
     const currentVersionId = FullModelLikeIdUtils.parse($stateParams.fullModelId).versionId;
     $scope.currentVersionId = currentVersionId;
+    $scope.warningMsg = '';
 
-    $scope.$watch("savedModel", function(nv) {
+    $scope.$watch("savedModel", function(nv, ov) {
         if (!nv) return;
 
         $scope.currentVersionIdx = nv.inlineVersions.findIndex(v => v.versionId === currentVersionId);
@@ -23,6 +25,10 @@ app.controller("_AgentSavedModelController", function($scope, $rootScope, $state
         $scope.currentVersionData = nv.inlineVersions[$scope.currentVersionIdx];
     });
 
+    // to trigger change detection in child components
+    $scope.currentVersionUpdateCounter = 0;
+    $scope.$watch('currentVersionData', Debounce().withDelay(200, 200).wrap(() => $scope.currentVersionUpdateCounter++), true);
+
     $scope.save = function() {
         if (!$scope.isDirty()) {
             return $q.when('agent not dirty');
@@ -62,6 +68,14 @@ app.controller("_AgentSavedModelController", function($scope, $rootScope, $state
     };
     $rootScope.saveAgentModel = $scope.save;
 
+    $scope.internalCodeEnvsHRef = function() {
+        if ($scope.appConfig.isAutomation) {
+            return $state.href("admin.codeenvs-automation.internal");
+        } else {
+            return $state.href("admin.codeenvs-design.internal");
+        }
+    }
+
     $scope.saveIgnoringQuickTestQuery = function() {
         if (!$scope.isDirty()) {
             return $q.when('agent not dirty');
@@ -76,7 +90,7 @@ app.controller("_AgentSavedModelController", function($scope, $rootScope, $state
 
     $scope.isConfigDirty = function() {
         let frankenVersion = angular.copy($scope.currentlySavedInlineModel);
-        frankenVersion["quickTestQuery"] = $scope.savedModel.inlineVersions[$scope.currentVersionIdx]["quickTestQuery"];
+        frankenVersion["quickTestQueryStr"] = $scope.savedModel.inlineVersions[$scope.currentVersionIdx]["quickTestQueryStr"];
 
         return !angular.equals(frankenVersion, $scope.savedModel.inlineVersions[$scope.currentVersionIdx]);
     };
@@ -85,15 +99,24 @@ app.controller("_AgentSavedModelController", function($scope, $rootScope, $state
         if (!$scope.savedModel) return false;
         return !angular.equals($scope.savedModel.inlineVersions[$scope.currentVersionIdx], $scope.currentlySavedInlineModel);
     };
+    $scope.getCurrentFile = function() {
+        if (!$scope.savedModel) return false;
+        return $scope.savedModel.id + "_" + $scope.currentVersionId + ".py";
+    };
     $rootScope.agentModelIsDirty = $scope.isDirty;
+    $rootScope.agentModelCurrentFile = $scope.getCurrentFile;
 
-    checkChangesBeforeLeaving($scope, $scope.isDirty);
+    function allowedTransitionsFn(data) {
+        return data.toState?.name?.startsWith('projects.project.savedmodels.savedmodel.agent') 
+            && data.fromState?.name?.startsWith('projects.project.savedmodels.savedmodel.agent') 
+            && data.toParams?.fullModelId === data.fromParams?.fullModelId;
+    }
+
+    checkChangesBeforeLeaving($scope, $scope.isDirty, null, allowedTransitionsFn);
 });
 
-app.controller("AgentSavedModelDesignController", function($scope, $rootScope, $state, $stateParams, $controller, ActivityIndicator, CodeBasedEditorUtils,
-                                                           CreateModalFromTemplate, DataikuAPI, PluginConfigUtils, PluginsService, TopNav) {
-    $controller("_AgentSavedModelController", {$scope});
-
+app.controller("AgentSavedModelDesignController", function($scope, $rootScope, $state, $stateParams, Debounce, ActivityIndicator, CodeBasedEditorUtils,
+                                                           CreateModalFromTemplate, DataikuAPI, PluginConfigUtils, PluginsService, TopNav, SmartId) {
     $scope.editorOptions = CodeBasedEditorUtils.editorOptions('text/x-python', $scope, true);
     CodeBasedEditorUtils.registerBroadcastSelectionHandler($scope.editorOptions);
     CodeBasedEditorUtils.setRecipeScript($scope.script);
@@ -133,20 +156,18 @@ app.controller("AgentSavedModelDesignController", function($scope, $rootScope, $
             $scope.availableLLMs = data.identifiers;
 
             if ($scope.savedModel.savedModelType === "TOOLS_USING_AGENT") {
-                // Don't allow selecting myself as the LLM for this agent, to avoid infinite loop
-                $scope.availableLLMs = $scope.availableLLMs.filter(llm => !llm.id.startsWith("agent:" + $scope.savedModel.id) && llm.type !== "RETRIEVAL_AUGMENTED");
+                $scope.availableLLMs = $scope.availableLLMs.filter(llm => (
+                    !llm.id.startsWith("agent:" + $scope.savedModel.id)  // Don't allow selecting myself as the LLM for this agent, to avoid infinite loop
+                    && llm.type !== "RETRIEVAL_AUGMENTED"
+                    && (
+                        $scope.currentVersionData.toolsUsingAgentSettings.allowAgentsAsLLM 
+                        || (llm.type !== "SAVED_MODEL_AGENT")
+                    )
+                ));
                 $scope.onToolLLMChange();
             }
         }).error(setErrorInScope.bind($scope));
 
-        $scope.internalCodeEnvsHRef = function() {
-            if ($scope.appConfig.isAutomation) {
-                return $state.href("admin.codeenvs-automation.internal");
-            } else {
-                return $state.href("admin.codeenvs-design.internal");
-            }
-        }
-
         if (!$scope.noSetLoc) {
             if ($scope.savedModel.savedModelType === "PLUGIN_AGENT"){
                 TopNav.setLocation(TopNav.TOP_GENAI_MODELS, TopNav.LEFT_GENAI_MODELS, "PLUGIN_AGENT-SAVED_MODEL-VERSION", "design");
@@ -156,6 +177,20 @@ app.controller("AgentSavedModelDesignController", function($scope, $rootScope, $
         }
     });
 
+    $scope.$watch('currentVersionData.toolsUsingAgentSettings.tools', Debounce().withDelay(500, 500).wrap(() => {
+        if ($scope.availableTools && $scope.currentVersionData.toolsUsingAgentSettings.tools.length) {
+            const kbTools = $scope.availableTools.filter(tool => tool.type === 'VectorStoreSearch').map(tool => SmartId.fromTor(tool));
+            $scope.hasKbTools = $scope.currentVersionData.toolsUsingAgentSettings.tools.some(tool => !tool.disabled && kbTools.includes(tool.toolRef));
+        }
+    }), true);
+
+    $scope.isToolAvailable = function(usedTool) {
+        if ($scope.availableTools && usedTool?.toolRef) {
+            return $scope.availableTools.some((l) => SmartId.fromTor(l) === usedTool.toolRef);
+        }
+        return false;
+    };
+
     $scope.createNewTool = function ($event) {
         $event.preventDefault();
         const stateName = 'projects.project.agenttools.list';
@@ -163,39 +198,63 @@ app.controller("AgentSavedModelDesignController", function($scope, $rootScope, $
             $scope.save().then(function () {
                 ActivityIndicator.success("Saved");
                 $scope.$state.go(stateName, { createTool: true });
-            }).error(setErrorInScope.bind($scope));
+            }).catch(setErrorInScope.bind($scope));
         } else {
             $scope.$state.go(stateName, { createTool: true });
         }
     };
 
-    $scope.showToolDescriptionModal = function(usedTool){
-        // support shared tools
-        let toolProjectKey = $scope.savedModel.projectKey;
-        let toolId = usedTool.toolRef;
-        if (usedTool.toolRef.includes('.')) {
-            [toolProjectKey, toolId] = usedTool.toolRef.split('.');
-        }
+    $scope.toggleTool = function(usedTool) {
+        usedTool.disabled = !usedTool.disabled;
+    };
 
-        DataikuAPI.agentTools.get(toolProjectKey, toolId).success(agentTool => {
-            DataikuAPI.agentTools.getDescriptor(agentTool.projectKey, agentTool.id).success(descriptor => {
-                const modalScope = $scope.$new(true);
-                modalScope.toolName = agentTool.name;
-                // remove the global description that was appended by the back-end in get-descriptor to show it in another field
-                modalScope.internalDescription = descriptor.description.replace(agentTool.additionalDescriptionForLLM, "").trimEnd();
-                modalScope.globalDescription = agentTool.additionalDescriptionForLLM;
-                modalScope.agentSpecificDescription = usedTool.additionalDescription || "";
-                modalScope.agentToolURL = $state.href("projects.project.agenttools.agenttool", {projectKey: agentTool.projectKey, agentToolId: agentTool.id});
-                CreateModalFromTemplate('/templates/savedmodels/agents/agent-tool-description-modal.html', modalScope);
-            }).error(setErrorInScope.bind($scope));
+    $scope.editToolDescriptionModal = function(usedTool) {
+        if (!usedTool.toolRef) {
+            return;
+        }
+        const smartId = SmartId.resolve(usedTool.toolRef, $stateParams.projectKey);
+
+        DataikuAPI.agentTools.get(smartId.projectKey, smartId.id).success(agentTool => {
+            const modalScope = $scope.$new(true);
+            modalScope.tool = agentTool;
+            modalScope.usedTool = usedTool;
+            modalScope.toolAdditionalDescription = usedTool.additionalDescription;
+
+            CreateModalFromTemplate("/templates/savedmodels/agents/agent-design-edit-tool-description-modal.html", modalScope, null, function(modalScope) {
+                modalScope.accept = function() {
+                    if (modalScope.toolAdditionalDescription) {
+                        usedTool.additionalDescription = modalScope.toolAdditionalDescription;
+                    } else {
+                        delete usedTool.additionalDescription;
+                    }
+                };
+
+                modalScope.showToolDescriptionModal = function() {
+                    DataikuAPI.agentTools.getDescriptor(agentTool.projectKey, agentTool.id).success(descriptor => {
+                        const innerModalScope = $scope.$new(true);
+                        innerModalScope.toolName = agentTool.name;
+                        // remove the global description that was appended by the back-end in get-descriptor to show it in another field
+                        innerModalScope.internalDescription = descriptor.description.replace(agentTool.additionalDescriptionForLLM, "").trimEnd();
+                        innerModalScope.globalDescription = agentTool.additionalDescriptionForLLM;
+                        innerModalScope.agentSpecificDescription = modalScope.toolAdditionalDescription || "";
+                        innerModalScope.agentToolURL = $state.href("projects.project.agenttools.agenttool", {projectKey: agentTool.projectKey, agentToolId: agentTool.id});
+                        CreateModalFromTemplate('/templates/savedmodels/agents/agent-tool-description-modal.html', innerModalScope);
+                    }).error(setErrorInScope.bind($scope));
+                };
+            });
         }).error(setErrorInScope.bind($scope));
     }
 });
 
-app.controller("AgentSavedModelSettingsController", function($scope, $controller, $stateParams, DataikuAPI, TopNav) {
-    $controller("_AgentSavedModelController", { $scope });
+app.controller("AgentSavedModelSettingsController", function($scope, $controller, $stateParams, DataikuAPI, TopNav, FLOW_COMPUTABLE_DEPENDENCIES, SavedModelHelperService) {
+    $scope.availableDependencies = undefined;
+    $scope.currentDependencies = undefined;
 
-    $scope.$watch("savedModel", (nv) => {
+    $scope.isDependenciesEnabled = function(){
+        return $scope.savedModel?.savedModelType === 'PYTHON_AGENT';
+    }
+
+    $scope.$watch("savedModel", (nv, ov) => {
         if (!nv) return;
 
         switch ($scope.savedModel.savedModelType) {
@@ -216,6 +275,12 @@ app.controller("AgentSavedModelSettingsController", function($scope, $controller
                     $scope.activeLLM = data.identifiers.find(l => l.id === $scope.currentAgentSettings.llmId);
                 }
             }).error(setErrorInScope.bind($scope));
+
+            DataikuAPI.codeenvs.checkAgentsCodeEnv($stateParams.projectKey)
+                .then(function ({data}) {
+                    $scope.codeEnvType = data["codeEnvType"];
+                    $scope.codeEnvError = data["codeEnvError"];
+                }).catch(setErrorInScope.bind($scope));
         }
 
         if (!$scope.noSetLoc) {
@@ -225,11 +290,52 @@ app.controller("AgentSavedModelSettingsController", function($scope, $controller
                 TopNav.setLocation(TopNav.TOP_GENAI_MODELS, TopNav.LEFT_GENAI_MODELS, "AGENT-SAVED_MODEL-VERSION", "settings");
             }
         }
+
+        $scope.addDependency = function(dependencyToAdd){
+            const pythonSettings = $scope.currentVersionData.pythonAgentSettings;
+            if (!pythonSettings.dependencies) {
+                pythonSettings.dependencies = [];
+            }
+
+            const isAlreadyAdded = pythonSettings.dependencies.some(dep => dep.ref === dependencyToAdd.smartId);
+            if (isAlreadyAdded) {
+                $scope.warningMsg = "Object already added as dependency.";
+                return;
+            }
+
+            const newDependency = {
+                type: dependencyToAdd.model?.savedModelType || dependencyToAdd.type,
+                ref: dependencyToAdd.smartId
+            };
+
+            pythonSettings.dependencies.push(newDependency);
+            $scope.currentDependencies.push(dependencyToAdd);
+            $scope.warningMsg = "";
+        };
+
+        $scope.removeDependency = function(index) {
+            if (index < 0) { return; }
+            const pythonSettings = $scope.currentVersionData.pythonAgentSettings;
+            if (pythonSettings?.dependencies) {
+                pythonSettings.dependencies.splice(index, 1);
+                $scope.currentDependencies.splice(index, 1);
+            }
+        };
+
+        // only load this once as it depends on the project not the model
+        if ($scope.isDependenciesEnabled() && $scope.availableDependencies === undefined) {
+            $scope.availableDependencies = [];
+            $scope.currentDependencies = [];
+            DataikuAPI.taggableObjects.listAccessibleObjects($stateParams.projectKey).then(function(resp) {
+                const dependencies = resp.data;
+                $scope.currentDependencies = SavedModelHelperService.enrichDependencies($scope.currentAgentSettings?.dependencies, dependencies)
+                $scope.availableDependencies = SavedModelHelperService.filterDependencies(dependencies, FLOW_COMPUTABLE_DEPENDENCIES);
+            }).catch(setErrorInScope.bind($scope));
+        }
     });
 });
 
 app.controller("AgentVersionHistoryController", function($scope, $stateParams, $state, $controller, TopNav) {
-    $controller("_AgentSavedModelController", {$scope});
     $scope.redirectCallback = function() {
         $state.go('projects.project.savedmodels.savedmodel.versions', {projectKey:$stateParams.projectKey, savedModelId:$scope.savedModel.id});
     };
@@ -238,8 +344,6 @@ app.controller("AgentVersionHistoryController", function($scope, $stateParams, $
 
 
 app.controller("AgentSavedModelLogsController", function($scope, $stateParams, $controller, DataikuAPI, Dialogs, TopNav) {
-    $controller("_AgentSavedModelController", { $scope });
-
     $scope.listLogs = function() {
         DataikuAPI.savedmodels.agents.listLogs($stateParams.projectKey, $scope.savedModel.id, $scope.currentVersionId).success(function(data) {
             $scope.agentLogs = data;
@@ -257,16 +361,23 @@ app.controller("AgentSavedModelLogsController", function($scope, $stateParams, $
         });
     }
 
-    $scope.logDelete = function(projectKey, savedModelId, versionId, logName) {
+    $scope.deleteLog = function(projectKey, savedModelId, versionId, logName) {
         return DataikuAPI.savedmodels.agents.deleteLog(projectKey, savedModelId, versionId, logName)
                 .error(setErrorInScope.bind($scope))
                 .finally($scope.listLogs);
     }
 
-    $scope.logGet = DataikuAPI.savedmodels.agents.getLog;
-    $scope.logDownloadURL = function(projectKey, savedModelId, versionId, logName) {
-        return "/dip/api/savedmodels/agents/stream-log?projectKey=" + projectKey + "&smId=" + encodeURIComponent(savedModelId) + "&versionId=" + encodeURIComponent(versionId) + "&logName=" + encodeURIComponent(logName);
-    }
+    $scope.getLog = DataikuAPI.savedmodels.agents.getLog;
+
+    $scope.logDownloadURL = (projectKey, savedModelId, versionId, logName) => {
+        const params = new URLSearchParams({
+            projectKey,
+            smId: savedModelId,
+            versionId,
+            logName
+        });
+        return `/dip/api/savedmodels/agents/stream-log?${params}`;
+    };
 
     $scope.$watch("savedModel", (nv) => {
         if (!nv) return;
@@ -290,7 +401,7 @@ app.component('agentArtifacts', {
     },
     templateUrl: '/templates/savedmodels/agents/artifacts.html',
 
-    controller: function($scope) {
+    controller: function() {
         const $ctrl = this;
 
         $ctrl.$onInit = function() {
@@ -303,9 +414,65 @@ app.component('agentArtifacts', {
             // If the agent returns no results, we could have a sources section with empty `items`
             return $ctrl.artifacts?.some(artifact => artifact.parts.length > 0);
         };
+
+        $ctrl.shouldExpandArtifact = function(artifact) {
+            // Always expand artifacts that do not have a hierarchy field
+            // Expand thinking artifacts to level 1
+            // Expand all other artifacts to level 2
+            if (!artifact.hierarchy || !artifact.hierarchy.length) {
+                return true;
+            }
+            if (artifact.type === "REASONING") {
+                return artifact.hierarchy.length <= 1;
+            }
+            return artifact.hierarchy.length <= 2;
+        };
     }
 });
 
+app.component('agentArtifact', {
+    bindings: {
+        artifact: '<',
+        expanded: '<',
+    },
+    templateUrl: '/templates/savedmodels/agents/artifact.html',
+
+    controller: function($dkuSanitize, Logger) {
+        const $ctrl = this;
+
+        $ctrl.$onInit = function() {
+            if ($ctrl.expanded !== false) {
+                $ctrl.expanded = true;
+            }
+        };
+
+        $ctrl.convertToMarkdown = function(reasoningParts) {
+            for (let reasoning of reasoningParts) {
+                if (reasoning.type === "TEXT") {
+                    try {
+                        reasoning.reasoning  = $dkuSanitize(marked(reasoning.text, { breaks: true}));
+                    }
+                    catch(e) {
+                        Logger.error('Error parsing markdown HTML, switching to plain text', e);
+                    }
+                }
+            }
+            return reasoningParts;
+        };
+
+        $ctrl.hasHierarchy = function() {
+            return $ctrl.artifact.hierarchy && $ctrl.artifact.hierarchy.length;
+        };
+    },
+});
+
+app.component('agentHierarchy', {
+    bindings: {
+        hierarchy: '<',
+    },
+    templateUrl: '/templates/savedmodels/agents/hierarchy.html',
+});
+
 app.component('artifactRecordsTable', {
     bindings: {
         records: '<',
