@@ -10,7 +10,7 @@
      * Service responsible for displaying charts based on a pivot response.
      * (!) This directive previously was in static/dataiku/js/simple_report/chart_views.js
      */
-    app.directive('pivotChartResult', function($timeout, $q, $state, CHART_TYPES, DKUPivotCharts, Logger, ChartFeatures, ChartDimension, ChartZoomControlAdapter, CanvasUtils, EChartsManager, Fn, ChartStoreFactory, ChartFormattingPane, ChartFormattingPaneSections, ChartActivityIndicator, ChartDefinitionChangeHandler, DSSVisualizationThemeUtils) {
+    app.directive('pivotChartResult', function($timeout, $q, $state, CHART_TYPES, DKUPivotCharts, Logger, ChartFeatures, ChartDimension, ChartZoomControlAdapter, CanvasUtils, EChartsManager, Fn, ChartStoreFactory, ChartFormattingPane, ChartFormattingPaneSections, ChartActivityIndicator, ChartDefinitionChangeHandler, DSSVisualizationThemeUtils, ChartDrilldown, DRILL_UP_SOURCE) {
 
         return {
             templateUrl: '/static/dataiku/js/simple_report/directives/pivot-chart-result/pivot-chart-result.directive.html',
@@ -32,6 +32,10 @@
                     }
                 };
 
+                scope.breadcrumbDrillUp = (targetDimension) => {
+                    ChartDrilldown.handleMultiLevelDrillUp(scope.chart.def, targetDimension, DRILL_UP_SOURCE.BREADCRUMB);
+                };
+
                 // Chart lazy loading causes issues with dashboard export, so we disable it when in an export flow.
                 scope.disableLazyLoading = getCookie('dku_unattended') === 'true';
 
@@ -84,6 +88,14 @@
 
                     const { store, id } = ChartStoreFactory.getOrCreate(scope.chart.def.$chartStoreId);
                     scope.chart.def.$chartStoreId = id;
+
+                    const displayBreadcrumb = ChartFeatures.shouldDisplayBreadcrumb(scope.chart.def)
+                        && scope.breadcrumbs && scope.breadcrumbs.some(b => !!b.elements.length)
+                        && !scope.noBreadcrumb;
+                    if (displayBreadcrumb) {
+                        element.find('[data-qa-chart-breadcrumb]').css('display', '');
+                    }
+
                     const rootElement = getRootElement(scope.chart.def);
                     const dims = ChartDimension.getChartDimensions(scope.chart.def);
                     const axesDef = ChartDimension.getAxesDef(scope.chart.def, dims, store);
@@ -92,6 +104,7 @@
                     if (rootElement && scope.chart && scope.chart.theme && scope.chart.theme.generalFormatting && scope.chart.theme.generalFormatting.fontFamily) {
                         element.css('--visualization-font-family', scope.chart.theme.generalFormatting.fontFamily);
                     }
+
                     switch (scope.chart.def.type) {
                         case CHART_TYPES.GROUPED_COLUMNS: {
                             DKUPivotCharts.GroupedColumnsChart(rootElement, scope.chart.def, scope, axesDef, scope.response.result.pivotResponse);
@@ -611,7 +624,7 @@
                             y = parseFloat($foreignObject.attr('y')),
                             width = parseFloat($foreignObject.attr('width')),
                             height = parseFloat($foreignObject.attr('height'));
-                        const origCanvas = $foreignObject.find('canvas').get(0);
+                        const origCanvas = $foreignObject.find('.canvas-export').get(0);
                         canvas.getContext('2d').drawImage(origCanvas, (x + horizontalOffset) * scale, (y + verticalOffset) * scale, width * scale, height * scale);
                     }
 
@@ -624,7 +637,7 @@
                     function addLegendToCanvas(chartElement, canvas, scale, verticalOffset) {
                         const d = $q.defer();
                         const $legendDiv = element.find('.legend-zone');
-                        if ($legendDiv.size() === 0) {
+                        if ($legendDiv.size() === 0 || $legendDiv.width() === 0) {
                             d.resolve();
                         } else {
                             const legendOffset = $legendDiv.offset();
@@ -805,7 +818,7 @@
                     };
 
                     const allCanvas = [CanvasUtils.svgToCanvas(svg1.get(0), svg1.width(), svg1.height(), options)];
-                    if (scope.chart.def.boxplotBreakdownDim.length) {
+                    if (scope.chart.def.boxplotBreakdownDim.length || scope.chart.def.boxplotBreakdownHierarchyDimension.length) {
                         allCanvas.push(CanvasUtils.svgToCanvas(svg2.get(0), svg2.width(), svg2.height(), options));
                     }
                     $q.all(allCanvas).then(subcanvas => {
