@@ -13,8 +13,11 @@
             'dropdownAlignRight': '<'               // boolean
         },
         templateUrl : '/static/dataiku/shared/components/object-edition-buttons/object-edition-buttons.component.html',
-        controller: function(CreateModalFromTemplate, $rootScope) {
+        controller: function(CreateModalFromTemplate, $rootScope, translate) {
             const ctrl = this;
+
+            ctrl.translate = translate;
+    
             let unwatchProjectSummary;
 
             ctrl.$onInit = function() {
@@ -44,12 +47,12 @@
                     ctrl.canWrite = ctrl.canWriteOverride || false;
                     return;
                 }
-                
+
                 ctrl.canSaveWithCustomCommitMessage = $rootScope.projectSummary.commitMode !== 'ALL_EXPLICIT' && (ctrl.objectType !== 'RECIPE' || $rootScope.projectSummary.commitMode !== 'RECIPES_EXPLICIT');
                 ctrl.canCommit = $rootScope.projectSummary.commitMode === 'ALL_EXPLICIT' || (ctrl.objectType === 'RECIPE' && $rootScope.projectSummary.commitMode === 'RECIPES_EXPLICIT');
                 ctrl.canWrite = $rootScope.projectSummary.canWriteProjectContent || ctrl.canWriteOverride;
             };
-    
+
             ctrl.saveWithCustomCommitMessage = function () {
                 CreateModalFromTemplate("/templates/git/commit-message-only-modal.html", $rootScope.$new(), null, function(newScope) {
                     newScope.commitData = {};
@@ -57,7 +60,7 @@
                     if (ctrl.currentSaveCommitMessage) {
                         newScope.commitData.message = ctrl.currentSaveCommitMessage;
                     }
-    
+
                     newScope.commit = function(){
                         newScope.resolveModal(newScope.commitData);
                     };
@@ -66,7 +69,7 @@
                     ctrl.save(commitData.message);
                 });
             };
-    
+
             ctrl.commit = function() {
                 CreateModalFromTemplate("/templates/git/commit-object-modal.html", $rootScope.$new(), null, function(newScope) {
                     newScope.object = {
@@ -77,5 +80,5 @@
             };
         }
     });
-    
+
 })();
