@@ -15,7 +15,6 @@ import json
 import logging
 import sys
 import time
-from collections import defaultdict
 
 import numpy as np
 from sklearn.model_selection import train_test_split
@@ -27,8 +26,7 @@ from dataiku.base.utils import safe_unicode_str
 from dataiku.core import dkujson
 from dataiku.core import doctor_constants
 from dataiku.core import intercom
-from dataiku.core.dku_pandas_csv import dataframe_to_csv
-from dataiku.core.doctor_constants import TARGET_VARIABLE
+from dataiku.core.doctor_constants import TARGET_VARIABLE, TIME_VARIABLE
 from dataiku.core.percentage_progress import PercentageProgress
 from dataiku.doctor import step_constants
 from dataiku.doctor import utils
@@ -58,8 +56,9 @@ from dataiku.doctor.prediction.common import regridify_optimized_params
 from dataiku.doctor.prediction.common import PredictionAlgorithmNaNSupport
 from dataiku.doctor.prediction.metric import MAPE
 from dataiku.doctor.timeseries.models import TimeseriesForecastingAlgorithm
+from dataiku.doctor.timeseries.posttraining.interactive_scenarios_handler import TimeseriesInteractiveScenariosHandler
 from dataiku.doctor.timeseries.preparation.preprocessing import TimeseriesPreprocessing, get_windows_list, \
-    add_rolling_windows
+    add_rolling_windows_for_training, add_rolling_windows_for_scoring
 from dataiku.doctor.timeseries.score.scoring_handler import resample_for_scoring
 from dataiku.doctor.timeseries.train.training_handler import resample_for_training
 from dataiku.doctor.utils.gpu_execution import log_nvidia_smi_if_use_gpu
@@ -91,10 +90,8 @@ from dataiku.doctor.utils.split import load_test_set
 from dataiku.doctor.utils.split import load_train_set
 from dataikuscoring.utils.prediction_result import ClassificationPredictionResult
 from dataikuscoring.utils.prediction_result import PredictionResult
-from dataiku.doctor.timeseries.train.split_handler import TimeseriesInteractiveScoringSplitHandler
 from dataiku.doctor.timeseries.preparation.preprocessing import get_external_features
-from dataiku.doctor.timeseries.utils import timeseries_iterator, str_to_hash, ModelForecast, build_quantile_column_name, \
-    FORECAST_COLUMN
+from dataiku.doctor.timeseries.utils import ModelForecast
 from dataiku.doctor.timeseries.perf.model_perf import TimeseriesModelScorer
 from dataiku.doctor.timeseries.utils import get_dataframe_of_timeseries_identifier, SINGLE_TIMESERIES_IDENTIFIER
 
@@ -922,27 +919,27 @@ def compute_timeseries_residuals(job_id, split_desc, core_params, preprocessing_
     preprocessing_params = preprocessing_folder_context.read_json("rpreprocessing_params.json")
     listener = ProgressListener()
     modeling_params = model_folder_context.read_json("rmodeling_params.json")
-    full_timeseries_preprocessing = TimeseriesPreprocessing(preprocessing_folder_context, core_params, preprocessing_params, modeling_params, listener)
+    algorithm = TimeseriesForecastingAlgorithm.build(modeling_params["algorithm"])
+    full_timeseries_preprocessing = TimeseriesPreprocessing(preprocessing_folder_context, core_params, preprocessing_params, modeling_params, listener, algorithm.SUPPORTS_EXTERNAL_FEATURES)
     split_folder_context = build_folder_context(split_folder)
     full_df = load_train_set(core_params, preprocessing_params, split_desc, "full",
                    split_folder_context, use_diagnostics=False)
 
     resolved_params = model_folder_context.read_json("actual_params.json")["resolved"]
     optimized_modeling_params = regridify_optimized_params(resolved_params, modeling_params)
-    algorithm = TimeseriesForecastingAlgorithm.build(modeling_params["algorithm"])
     full_df = resample_for_training(
         full_df,
         split_desc["schema"],
         preprocessing_params[doctor_constants.TIMESERIES_SAMPLING],
         core_params,
         preprocessing_params,
-        modeling_params["isShiftWindowsCompatible"],
+        modeling_params.get("isShiftWindowsCompatible", False),
         modeling_params["metrics"]["evaluationMetric"] == MAPE.name,
     )
 
     # Windowing is performed once and for all on the full dataframe
     windows_list = get_windows_list(preprocessing_params) if modeling_params.get("isShiftWindowsCompatible", False) else []
-    full_df = add_rolling_windows(full_df, core_params, windows_list)
+    full_df = add_rolling_windows_for_training(full_df, core_params, windows_list, preprocessing_params, preprocessing_folder_context)
 
     transformed_full_df = full_timeseries_preprocessing.fit_and_process(
         full_df,
@@ -965,6 +962,12 @@ def compute_timeseries_interactive_scoring_creation(job_id, split_desc, core_par
                                  modellike_folder=None, computation_parameters=None, postcompute_folder=None,
                                  train_split_desc=None, train_split_folder=None):
 
+    computation_parameters = computation_parameters or {}
+    target_timeseries_identifier = computation_parameters.get('timeseriesIdentifier')
+    source_scenario_id = computation_parameters.get('scenarioId')
+    start_date_str = computation_parameters.get('startDate')
+    scenario_name = computation_parameters.get('scenarioName')
+
     model_folder_context = build_folder_context(model_folder)
     preprocessing_folder_context = build_folder_context(preprocessing_folder)
     preprocessing_params = preprocessing_folder_context.read_json("rpreprocessing_params.json")
@@ -973,8 +976,9 @@ def compute_timeseries_interactive_scoring_creation(job_id, split_desc, core_par
                              split_folder_context, use_diagnostics=False)
 
     modeling_params = model_folder_context.read_json("rmodeling_params.json")
+    resolved_params = model_folder_context.read_json("actual_params.json")["resolved"]
 
-    external_features = get_external_features(preprocessing_params, modeling_params["isShiftWindowsCompatible"])
+    external_features = get_external_features(preprocessing_params, modeling_params.get("isShiftWindowsCompatible", False))
     if len(external_features) == 0:
         raise ValueError("The model doesn't use any external features.")
     full_df = resample_for_scoring(
@@ -990,37 +994,12 @@ def compute_timeseries_interactive_scoring_creation(job_id, split_desc, core_par
 
     # We retrieve only identifiers that we trained on
     trained_models_identifiers = list(model_folder_context.read_json("forecasts.json.gz")["perTimeseries"].keys())
-
-
-    interactive_scoring_split_handler = TimeseriesInteractiveScoringSplitHandler(core_params)
-    _, scenario_df, _ = next(interactive_scoring_split_handler.split(full_df))
-
-    feature_columns = [core_params[doctor_constants.TIME_VARIABLE]] + core_params[doctor_constants.TIMESERIES_IDENTIFIER_COLUMNS] + external_features
-    scenario_df = scenario_df[feature_columns]
-    scenario_df['scenario_id'] = np.zeros(len(scenario_df), dtype=int)
-
-    interactive_scoring_model_folder_context = model_folder_context.get_subfolder_context("interactive-scoring-analysis")
-    interactive_scoring_model_folder_context.create_if_not_exist()
-
-    identifier_columns = core_params[doctor_constants.TIMESERIES_IDENTIFIER_COLUMNS]
-    identifiers_mapping = {}
-
-    for idx, (identifier_values, scenario_df_of_timeseries_identifier) in enumerate(timeseries_iterator(scenario_df, identifier_columns), 1):
-        if identifier_values not in trained_models_identifiers:
-            continue
-        csv_key = str_to_hash(identifier_values)
-        identifiers_mapping[identifier_values] = csv_key
-
-        # Remove identifier columns and keep only time, external features and scenario_id
-        columns_to_keep = [core_params[doctor_constants.TIME_VARIABLE]] + get_external_features(preprocessing_params) + ['scenario_id']
-        scenario_df_without_identifiers = scenario_df_of_timeseries_identifier[columns_to_keep]
-
-        filename = "scenario-%s.csv" % csv_key
-        with interactive_scoring_model_folder_context.get_file_path_to_write(filename) as predicted_file:
-            dataframe_to_csv(scenario_df_without_identifiers, predicted_file, open, header=True)
-
-    interactive_scoring_model_folder_context.write_json("identifiers_mapping.json", identifiers_mapping)
-
+    if target_timeseries_identifier in trained_models_identifiers:
+        interactive_scenarios_handler = TimeseriesInteractiveScenariosHandler(model_folder_context, resolved_params, preprocessing_params, core_params, full_df)
+        interactive_scenarios_handler.create_scenario(target_timeseries_identifier, source_scenario_id, pd.to_datetime(start_date_str), scenario_name)
+        interactive_scenarios_handler.dump_identifiers_mapping()
+    else:
+        logger.info("No model for timeseries identifier %s." % target_timeseries_identifier)
     return "ok"
 
 
@@ -1032,13 +1011,14 @@ def compute_timeseries_interactive_scoring_computation(job_id, split_desc, core_
     preprocessing_params = preprocessing_folder_context.read_json("rpreprocessing_params.json")
     split_folder_context = build_folder_context(split_folder)
     modeling_params = model_folder_context.read_json("rmodeling_params.json")
+    resolved_params = model_folder_context.read_json("actual_params.json")["resolved"]
+
     metrics_params = modeling_params["metrics"]
 
-    interactive_scoring_split_handler = TimeseriesInteractiveScoringSplitHandler(core_params)
     listener = ProgressListener()
-    timeseries_preprocessing = TimeseriesPreprocessing(preprocessing_folder_context, core_params, preprocessing_params, modeling_params, listener)
-    clf = from_pkl(model_folder_context)
     algorithm = TimeseriesForecastingAlgorithm.build(modeling_params["algorithm"])
+    timeseries_preprocessing = TimeseriesPreprocessing(preprocessing_folder_context, core_params, preprocessing_params, modeling_params, listener, algorithm.SUPPORTS_EXTERNAL_FEATURES)
+    clf = from_pkl(model_folder_context)
     fit_before_predict = algorithm.should_fit_before_predict()
 
     interactive_scoring_model_folder_context = model_folder_context.get_subfolder_context("interactive-scoring-analysis")
@@ -1046,7 +1026,7 @@ def compute_timeseries_interactive_scoring_computation(job_id, split_desc, core_
 
     full_df = load_train_set(core_params, preprocessing_params, split_desc, "full",
                              split_folder_context, use_diagnostics=False)
-    external_features = get_external_features(preprocessing_params, modeling_params["isShiftWindowsCompatible"])
+    external_features = get_external_features(preprocessing_params, modeling_params.get("isShiftWindowsCompatible", False))
     full_df = resample_for_scoring(
         full_df,
         split_desc["schema"],
@@ -1060,83 +1040,69 @@ def compute_timeseries_interactive_scoring_computation(job_id, split_desc, core_
 
     # Windowing is performed once and for all on the full dataframe
     windows_list = get_windows_list(preprocessing_params) if modeling_params.get("isShiftWindowsCompatible", False) else []
-    full_df = add_rolling_windows(full_df, core_params, windows_list)
+    full_df = add_rolling_windows_for_training(full_df, core_params, windows_list, preprocessing_params, preprocessing_folder_context)
 
-    external_features = get_external_features(preprocessing_params)
-    external_features_dtypes = {ef: full_df[ef].dtype for ef in external_features}
+    interactive_scenarios_handler = TimeseriesInteractiveScenariosHandler(model_folder_context, resolved_params, preprocessing_params, core_params, full_df)
 
-    timeseries_identifiers = computation_parameters['timeseriesIdentifiers']
+    identifier = computation_parameters['timeseriesIdentifier']
 
-    identifiers_mapping = interactive_scoring_model_folder_context.read_json("identifiers_mapping.json")
-    if interactive_scoring_model_folder_context.isfile("scenario-forecasts.json.gz"):
-        scenarios_results_by_identifiers = interactive_scoring_model_folder_context.read_json("scenario-forecasts.json.gz")
-    else:
-        scenarios_results_by_identifiers = {}
+    identifier_full_df = get_dataframe_of_timeseries_identifier(full_df, identifier)
+    preprocess_on_resampled_df = algorithm.USE_GLUON_TS
+    timeseries_preprocessing.load_resources()
+    timeseries_preprocessing.create_timeseries_preprocessing_handlers(identifier_full_df, preprocess_on_resampled_df, use_saved_resources=True)
 
-    for identifier in timeseries_identifiers:
-        scenarios_results_by_identifiers[identifier] = { "perScenarios": {} }
-        # Filter dataframe for current identifier
-        identifier_full_df = get_dataframe_of_timeseries_identifier(full_df, identifier)
-        _, test_df, historical_df = next(interactive_scoring_split_handler.split(identifier_full_df))
-        # Same as for the scoring recipe, avoid preprocessing failing with only NaN in target column, and future target values won't be used anyway to predict
-        # see TimeseriesScoringHandler::_split_past_and_future_df
-        test_df[core_params[TARGET_VARIABLE]] = 0
+    transformed_full_df = timeseries_preprocessing.process(
+        identifier_full_df,
+        step_constants.ProcessingStep.STEP_PREPROCESS_TRAIN,
+        preprocess_on_resampled_df
+    )
 
-        identifier_hash = identifiers_mapping.get(identifier)
-        with interactive_scoring_model_folder_context.get_file_path_to_read("scenario-%s.csv" % identifier_hash) as dataset_path:
-            scenario_df = pd.read_csv(dataset_path, sep="\t", dtype=external_features_dtypes)
+    date_format='%Y-%m-%dT%H:%M:%S.%fZ'
+    scenarios_metadata = interactive_scenarios_handler.get_scenarios_metadata(identifier)
+    for scenario_id, scenario_name in scenarios_metadata["names"].items():
+        scenario_df = interactive_scenarios_handler.get_scenario_df(identifier, scenario_id)
+        scenario_df[core_params[TARGET_VARIABLE]] = 0
+        scenario_df[core_params[TIME_VARIABLE]] = pd.to_datetime(scenario_df[core_params[TIME_VARIABLE]])
+        if identifier != SINGLE_TIMESERIES_IDENTIFIER:
+            for k, v in json.loads(identifier).items():
+                scenario_df[k] = v
+        for external_feature in external_features:
+            if external_feature not in scenario_df.columns:
+                scenario_df[external_feature] = pd.Series(None, index=scenario_df.index, dtype=full_df[external_feature].dtype)
 
-        scenarios_forecasts = {
-            "forecastTime": test_df[core_params[doctor_constants.TIME_VARIABLE]].astype(str).values,
-            "groundTruthTime": historical_df[core_params[doctor_constants.TIME_VARIABLE]][-max(2, len(scenario_df)):].astype(str).values,
-            "groundTruth": historical_df[core_params[doctor_constants.TARGET_VARIABLE]][-max(2, len(scenario_df)):].values,
-        }
+        # Process each scenario separately
+        historical_df_subset = transformed_full_df[pd.to_datetime(identifier_full_df[core_params[doctor_constants.TIME_VARIABLE]]) < np.datetime64(scenario_df[core_params[doctor_constants.TIME_VARIABLE]].iloc[0])]
 
-        preprocess_on_resampled_df = algorithm.USE_GLUON_TS
-        timeseries_preprocessing.load_resources()
-        timeseries_preprocessing.create_timeseries_preprocessing_handlers(historical_df, preprocess_on_resampled_df, use_saved_resources=True)
-
-        transformed_historical_df = timeseries_preprocessing.process(
-            historical_df,
-            step_constants.ProcessingStep.STEP_PREPROCESS_TRAIN,
+        # Process the updated_test_df for the current scenario
+        scenario_df = add_rolling_windows_for_scoring(scenario_df, core_params, windows_list, preprocessing_params, preprocessing_folder_context)
+        transformed_scenario_df = timeseries_preprocessing.process(
+            scenario_df,
+            step_constants.ProcessingStep.STEP_PREPROCESS_TEST,
             preprocess_on_resampled_df
         )
 
-        # Process each scenario separately
-        for scenario_id in scenario_df["scenario_id"].unique():
-            scenario_subset = scenario_df[scenario_df["scenario_id"] == scenario_id]
-            
-            # For each external feature column, update values from scenario
-            for col in external_features:
-                if col in scenario_subset.columns:
-                    test_df[col] = scenario_subset[col].values
+        model_scorer = TimeseriesModelScorer.build(core_params, metrics_params, use_external_features=True)
+        timeseries_forecasts = model_scorer.predict_all_test_timesteps(clf, historical_df_subset, transformed_scenario_df, fit_before_predict)[identifier]
+        forecasts = {
+            "forecast": timeseries_forecasts[ModelForecast.FORECAST_VALUES],
+            "forecastTime": timeseries_forecasts[ModelForecast.TIMESTAMPS],
+            "quantiles": []
+        }
+        for idx, q in enumerate(model_scorer.quantiles):
+            forecasts["quantiles"].append({
+                "forecast": timeseries_forecasts[ModelForecast.QUANTILES_FORECASTS][idx],
+                "quantile": q
+            })
+        serializable_forecasts = model_scorer.remove_nanif_in_one_forecast(forecasts)
+        scenarios_forecasts = {
+            "forecastTime": transformed_scenario_df[core_params[doctor_constants.TIME_VARIABLE]].astype(str).values,
+            "groundTruthTime": historical_df_subset[core_params[doctor_constants.TIME_VARIABLE]][-max(2, len(scenario_df)):].dt.strftime(date_format).values,
+            "groundTruth": historical_df_subset[core_params[doctor_constants.TARGET_VARIABLE]][-max(2, len(scenario_df)):].values,
+        }
+        scenarios_forecasts["forecast"] = serializable_forecasts["forecast"]
+        scenarios_forecasts["quantiles"] = serializable_forecasts["quantiles"]
+        interactive_scenarios_handler.write_scenarios(scenarios_forecasts, identifier, scenario_id)
 
-            # Process the updated_test_df for the current scenario
-            transformed_test_df = timeseries_preprocessing.process(
-                test_df,
-                step_constants.ProcessingStep.STEP_PREPROCESS_TEST,
-                preprocess_on_resampled_df
-            )
-
-            model_scorer = TimeseriesModelScorer.build(core_params, metrics_params, use_external_features=True)
-            timeseries_forecasts = model_scorer.predict_all_test_timesteps(clf, transformed_historical_df, transformed_test_df, fit_before_predict)[identifier]
-            forecasts = {
-                "forecast": timeseries_forecasts[ModelForecast.FORECAST_VALUES],
-                "forecastTime": timeseries_forecasts[ModelForecast.TIMESTAMPS],
-                "quantiles": []
-            }
-            for idx, q in enumerate(model_scorer.quantiles):
-                forecasts["quantiles"].append({
-                    "forecast": timeseries_forecasts[ModelForecast.QUANTILES_FORECASTS][idx],
-                    "quantile": q
-                })
-            serializable_forecasts = model_scorer.remove_nanif_in_one_forecast(forecasts)
-
-            scenarios_forecasts["forecast"] = serializable_forecasts["forecast"]
-            scenarios_forecasts["quantiles"] = serializable_forecasts["quantiles"]
-            scenarios_results_by_identifiers[identifier]["perScenarios"][str(scenario_id)] = scenarios_forecasts
-
-    interactive_scoring_model_folder_context.write_json("scenario-forecasts.json.gz", scenarios_results_by_identifiers)
     return "ok"
 
 
