@@ -16,40 +16,39 @@ from dataiku.llm.python.processing.completion_processor import CompletionProcess
 logger = logging.getLogger(__name__)
 
 
-def redact_data(data: dict, fields_to_redact: list[str]) -> None:
-    """ Recursively redacts specified fields in a nested dictionary or list by modifying the data structure *in place*.
-
-    Args:
-        data: The dictionary or list to process. This will be mutated.
-        fields_to_redact: A list of field names (keys) to redact.
-    """
-    if isinstance(data, dict):
-        for key, value in data.items():
-            if key in fields_to_redact:
-                data[key] = "**redacted**"
-            else:
-                redact_data(value, fields_to_redact)
-    elif isinstance(data, list):
-        for item in data:
-            redact_data(item, fields_to_redact)
-
-
-def plugin_get_redacted_command(command):
-    assert "config" in command
-    redacted_command = copy.deepcopy(command)
-
-    if password_params := command.get("pluginConfig", {}).get("dkuPasswordParams"):
-        redact_data(redacted_command["config"], password_params)
-        redact_data(redacted_command["pluginConfig"], password_params)
-        del redacted_command["pluginConfig"]["dkuPasswordParams"]
-
-    return redacted_command
-
-
-def get_redacted_command(command):
+def get_redacted_start_command(command):
     if "config" not in command:
         return command
-    return plugin_get_redacted_command(command)
+
+    def plugin_get_redacted_start_command(command):
+        assert "config" in command
+        redacted_command = copy.deepcopy(command)
+    
+        def redact_data(data: dict, fields_to_redact: list[str]) -> None:
+            """ Recursively redacts specified fields in a nested dictionary or list by modifying the data structure *in place*.
+        
+            Args:
+                data: The dictionary or list to process. This will be mutated.
+                fields_to_redact: A list of field names (keys) to redact.
+            """
+            if isinstance(data, dict):
+                for key, value in data.items():
+                    if key in fields_to_redact:
+                        data[key] = "**redacted**"
+                    else:
+                        redact_data(value, fields_to_redact)
+            elif isinstance(data, list):
+                for item in data:
+                    redact_data(item, fields_to_redact)
+    
+        if password_params := command.get("pluginConfig", {}).get("dkuPasswordParams"):
+            redact_data(redacted_command["config"], password_params)
+            redact_data(redacted_command["pluginConfig"], password_params)
+            del redacted_command["pluginConfig"]["dkuPasswordParams"]
+    
+        return redacted_command
+
+    return plugin_get_redacted_start_command(command)
 
 
 class PythonLLMServer:
@@ -64,14 +63,15 @@ class PythonLLMServer:
         self.run_counter = 1
 
     async def handler(self, command: StartAgentServerCommand):
-        logger.info("PythonLLMServer handler received command: %s", get_redacted_command(command))
 
         if command["type"] == "start-agent-server":
+            logger.info("PythonLLMServer handler received start command: %s", get_redacted_start_command(command))
             yield await asyncio.get_running_loop().run_in_executor(
                 self.executor, self.start, command
             )
 
         elif command["type"] == "process-completion-query":
+            logger.debug("PythonLLMServer handler received completion query command")
             logger.info("\n===============  Start completion query - run %s ===============", self.run_counter)
             try:
                 if command["stream"]:
@@ -96,6 +96,9 @@ class PythonLLMServer:
             if py_clazz == "dataiku.llm.python.tools_using.ToolUsingAgent":
                 from .tools_using import ToolsUsingAgent
                 clazz = ToolsUsingAgent
+            elif py_clazz == "dataiku.llm.python.tools_using_2.ToolUsingAgent":
+                from .tools_using_2 import ToolsUsingAgent
+                clazz = ToolsUsingAgent
             else:
                 raise Exception("Missing BaseLLM implementation")
 
