@@ -326,10 +326,14 @@ class RAGServer:
 
     @staticmethod
     def _extract_caller_security_tokens(process_prompt_command: ProcessSinglePromptCommand) -> List[str]:
-        if "query" not in process_prompt_command or "context" not in process_prompt_command["query"] or "callerSecurityTokens" not in process_prompt_command["query"]["context"]:
+        try:
+            raw_tokens = process_prompt_command["query"]["context"]["callerSecurityTokens"]
+        except (KeyError, TypeError):
             raise ValueError("Unable to process query: Retrieval-Augmented LLM with document-level security requires valid security tokens. Please ensure your security tokens are included with your query context.")
-        else:
-            return process_prompt_command["query"]["context"]["callerSecurityTokens"]
+        if not isinstance(raw_tokens, list) or not all(isinstance(token, str) for token in raw_tokens):
+            raise ValueError("Invalid format: 'callerSecurityTokens' must be a JSON Array of Strings")
+
+        return raw_tokens
 
     def _query_rewriter(self, relevant_messages: List[ChatMessage], question: str, trace: SpanBuilder) -> str:
         if self.search_input_strategy == "RAW_QUERY":
